/*
  Minimal models using Davis-Putnam
Heuristics:
1. fail whenever the wieght of a model is grater or equal then the last best
2. use positive clauses to infer a lower bound of the weight
2. all the variables whose weight is more than the current difference are negated

Note: it must hold weight*1000 < overflow.

Define:

USESIGNALS	compile with the functions for handling signals

*/

#include"dp.h"

#include<signal.h>
#include<sys/times.h>

#define PIDFILENAME "pid.mindp"

int *firstmodelofstate( state stt ) {
  int *p;
  int i;

  p=(int *) malloc( (stt->numvar+1)*sizeof(int) );
  if( p==NULL ) {
    printf("Malloc failed in firstmodelofstate()\n");
    exit(1);
  }

  for(i=1; i<=stt->numvar; i++)
    if( valueofvar( i, stt )!=0 ) 
      p[i]=valueofvar(i,stt);
    else 
      p[i]=-1;

  return p;
}

int nextmodelofstate( setofclauses cl, state stt, int *p ) {
  int i;
  int reversed;

  reversed=0;

  for(i=stt->numvar; i>=1; i--) 
    if( (valueofvar( i, stt )==0) && (cl->weight[i]==0) )
      if( p[i]==1 )
        p[i]=-1;
      else {
        p[i]=1;
        reversed=1;
        break;
      }

  if( reversed )
    return 1;
  else {
    free(p);
    return 0;
  }
}




                                /* signal handlers */

void onexit() {
  unlink(PIDFILENAME);
}

#ifdef USESIGNALS

  void usersignal( int s ) {
    printstatus();

    signal( SIGUSR1, usersignal);
  }

  void exitsignal( int s ) {
    printf("\nSignal %d caught. Exiting.\n",s);
    printstatus(s);
    onexit();

    exit(1);
  }
#endif


main() {
  int c;
  setofclauses cl;
  clauseindex ix;
  state stt;
  stack stk;
  int k;
  int temp;
  int *p;
  int cont,i;

  FILE *pidfile;
  int starttime, leaptime, endtime;


  cl=newsetofclauses();
  readclauses( stdin, &cl );

  #ifdef DEBUG
    printclauses( cl );
  #endif

  #ifdef USESIGNALS
    pidfile=fopen(PIDFILENAME,"w");
    fprintf(pidfile,"%d",getpid());
    fclose(pidfile);
  
    signal( SIGTERM, exitsignal );
    signal( SIGINT, exitsignal );
    signal( SIGUSR1, usersignal );
  #endif

  starttime=usertime();
  k=firstminweight( cl, &ix, &stt, &stk, DEFAULTDP );
  endtime=usertime();

  if( k<0 )
    printf("Inconsistente\n");
  else {
    printf("Consistente: peso=%d\n",k);

    for( p=firstmodelofstate(stt); cont; cont=nextmodelofstate( cl, stt, p ) ) {
      printf("[");
      for(i=1; i<=cl->numvar; i++)
        if(p[i]==1)
          printf("1");
        else
          printf("0");
      printf("]\n");
    }


    while( nextminweight(k, cl, ix, stt, stk, DEFAULTDP)!=-1 ) {
      cont=1;
      for( p=firstmodelofstate(stt); cont; cont=nextmodelofstate( cl, stt, p ) ) {
        printf("[");
        for(i=1; i<=cl->numvar; i++)
          if(p[i]==1)
            printf("1");
          else
            printf("0");
        printf("]\n");
      }
    }

  }

  fprintf(stderr,"%d\n",endtime-starttime);

  onexit();

  return 0;
}

