/*
  Minimal models using Davis-Putnam
Heuristics:
1. fail whenever the weight of a model is greater or equal than the last best found
2. use positive clauses to infer a lower bound of the weight
2. all the variables whose weight is more than the current difference are set to false

Note: it must hold weight*1000 < overflow.

Define:

USESIGNALS	compile with the functions for handling signals

*/

#include"dp.h"

#include<signal.h>

#define PIDFILENAME "pid.mindp"


                                /* signal handlers */

void onexit() {
  unlink(PIDFILENAME);
}

#ifdef USESIGNALS

  void usersignal( int s ) {
    printstatus();

    signal( SIGUSR1, usersignal);
  }

  void exitsignal( int s ) {
    printf("\nSignal %d caught. Exiting.\n",s);
    printstatus(s);
    onexit();

    exit(1);
  }
#endif


main( int argc, char *argv[] ) {
  int c;
  setofclauses cl;
  clauseindex ix;
  state stt;
  stack stk;
  int k;
  listofint l,r;
  int bound;

  FILE *pidfile;
  int starttime, endtime;

  if( argc<2 ) {
    printf("The bound must be given.\n");
    exit(1);
  }

  bound=atoi(argv[1]);

  cl=newsetofclauses();
  readclauses( stdin, &cl );

  /* printclauses( cl ); */

  #ifdef DEBUG
    printclauses( cl );
  #endif

  #ifdef USESIGNALS
    pidfile=fopen(PIDFILENAME,"w");
    fprintf(pidfile,"%d",getpid());
    fclose(pidfile);
  
    signal( SIGTERM, exitsignal );
    signal( SIGINT, exitsignal );
    signal( SIGUSR1, usersignal );
  #endif

  starttime=usertime();
  k=bounded( cl, &stt, bound, DEFAULTDP );
  endtime=usertime();

  if( k<0 )
    printf("Inconsistente\n");
  else
    printf("Consistente: peso=%d\n",k);

//  printf("%d\n",stt->branch);

//  fprintf(stderr,"nodes: %d\n",stt->branch);


  fprintf(stderr,"%4d ",endtime-starttime);

#ifdef NO
  /* r=reverselistofint(stt->intertime); */
  if( stt->intertime->next==NULL ) 
    fprintf(stderr,"%4d\n",endtime-starttime);
  else {
    fprintf(stderr,"%4d ",endtime - stt->intertime->elem );
    for(l=stt->intertime; l->next!=NULL; l=l->next)
      fprintf(stderr,"%4d ",l->elem - l->next->elem );
    fprintf(stderr,"\n");
  }

  fprintf(stderr,"%4d %4d ",k,k);
  for(l=stt->intermod; l!=NULL; l=l->next)
    fprintf(stderr,"%4d ",l->elem);
#endif

  fprintf(stderr,"\n");

  onexit();

  return 0;
}

