/*
  Choice of next literal to expand.

Define:
NUMPROP		set to -1 for non-propagating choice

DEBUG		print debug info

binary choice: set SECONDARY to propchoice(a,b,c,d,1)
*/

#include"list.h"
#include"queue.h"
#include"mindp.h"

queue *q;


				/* this is how up should run. Since we do not need any extra
				   feature here, 0 is ok */

#define UPOPTIONS	0


#define min(x,y) 		(((x)<(y))?(x):(y))


		/* how to combine positive and negative counts */

#define COMBINE(x,y)		(1024*(x)*(y)+(x)+(y))
/*
define COMBINE(x,y)		((x)+(y))
define COMBINE(x,y)		(y)
define COMBINE(x,y)		(((x)>(y))?(x):(y))
*/

		/* sign to choose */

#define CHOICESIGN(x,y)		(((x)>(y))?1:-1)
/*
define CHOICESIGN(x,y)	(-1)
*/

		/* number of literals to try to propagate in choice() */
		/* 1=occurrences in binary clauses. Do not use -1 or 0 */
/*
#define NUMPROP	1 
*/
/*
#define NUMPROP	cl->numvar-21*\
                (stk->elem[stk->stackindex].firstdeduced+stk->elem[stk->stackindex].numdeduced)
*/
#define NUMPROP		10-stk->stackindex


/******************************* Choice ***********************************/
/*
  Choice of the next literal to expand
*/


/*************************** Weight Choice ********************************/
/*
  Choose the next literal to expand according to the weight: the first is
the literal with maximal weight. The second is the next, in this order,
that is not currently assigned.
*/

int weightchoice( setofclauses cl, clauseindex ix, state stt, stack stk ) {
  int i;

  i=1;
  while( (i<=cl->numvar) && (cl->weight[ix->weightorder[i]]>0) ) {
    if( stt->val[ix->weightorder[i]]==0 )
      return ix->weightorder[i];
    i++;
  }
		/* all literals with weight >0 are already assigned */
  return 0;
}

/************************** End Weight Choice *****************************/


/**************************** Trivial Choice ******************************/
/*
  The next literal to expand is choosen accordingly to the variable order:
the first literal is 1. The second literal is 2 if it is unassigned, otherwise
is 3 if unassigned, and so on.
*/

int trivialchoice( setofclauses cl, clauseindex ix, state stt, stack stk ) {
  int i;

  for(i=1; i<=stt->numvar; i++)
    if( stt->val[i]==0 )
      return -i;

  fprintf(stderr,"Error: trivialchoice is not able to determine a literal.\n");
  exit(1);
}

/************************** End Trivial Choice ****************************/


/**************************** Shortest Choice *****************************/
/*
  Choose the variable that appear most often in the shortest clause.
Positive and negative occurences are combined. This procedure is not
optimal, as it check all clauses each time a choice of a new literal
is needed.
*/

int shortchoice( setofclauses cl, clauseindex ix, state stt, stack stk ) {
  int i,j,temp;
  int sizeshortest;
  int numshortest;
  int shortest[MAXCLAUSES];
  int c;
  int numactive;

  int positive[MAXVAR+1];
  int negative[MAXVAR+1];
  listofint l;

  int bestval,bestvar;



				/* determine the shortest clauses */
  sizeshortest=100000;
  for(c=0; c<cl->numclauses; c++)
    if( ! stt->truelits[c] ) {
      /* numactive++; */
      if( cl->cl[c].numlett - stt->falselits[c] < sizeshortest ) {
        sizeshortest= cl->cl[c].numlett - stt->falselits[c];
        shortest[0]=c;
        numshortest=1;
      }
      else if( cl->cl[c].numlett - stt->falselits[c] == sizeshortest ) {
        shortest[numshortest]=c;
        numshortest++;
      }
    }
				/* determine the number of occurences of variables */
  memset( positive, 0, (cl->numvar+1)*sizeof(int) );
  memset( negative, 0, (cl->numvar+1)*sizeof(int) );

  for(c=0; c<numshortest; c++)
    for(l=cl->cl[shortest[c]].lett; l!=NULL; l=l->next)
      if( stt->val[abs(l->elem)] == 0 )
        if( l->elem > 0 )
          positive[l->elem]++;
        else
          negative[-l->elem]++;


				/* return the maximum */
  bestval=-1;
  for(i=1; i<=cl->numvar; i++)
    if( stt->val[i]==0 )
      if( COMBINE( positive[i], negative[i] ) > bestval ) {
        bestval=COMBINE( positive[i], negative[i] );
        bestvar=CHOICESIGN( positive[i], negative[i] )*i;
      }

  return bestvar;
}

/************************** End Shortest Choice ***************************/


/************************** Propagation Choice ****************************/
/*
  Maximize the number of propagated literals, or the binary clauses
produced by unit propagating each literal among the best ones according
to the binary count.
maxbin==0 -> maximize the number of implied literals
maxbin==1 -> maximize the number of produced binary clauses
*/

int propchoice( setofclauses cl, clauseindex ix, state stt, stack stk, int maxbin ) {
  int k;
  int top[MAXVAR];
  int i,j;
  int temp;
  int bestval,bestvar;
  int pos,neg;
  int bin;
  int u;

				/* number of literals to propagate */
  k=NUMPROP;
  if( k<1 )
    k=1;

  #ifdef DEBUG
    printf("ChoiceK=%d Binary=%d\n",k,stt->binary); fflush(NULL);
  #endif

				/* determine the top k variables */

  if( k>=1 ) {

    for(i=0; i<cl->numvar; i++)
      top[i]=i+1;

    for(i=0; i<k; i++) {
				/* determine the i-th best */
      for(j=i+1; j<cl->numvar; j++)
        if( (
              ( COMBINE(stt->binarypos[top[i]],stt->binaryneg[top[i]]) <
	        COMBINE(stt->binarypos[top[j]],stt->binaryneg[top[j]]) ) &&
	      ( stt->val[top[j]]==0 )
            ) ||
            ( ( stt->val[top[i]]!=0 ) && ( stt->val[top[j]]==0) )
          ) {
          temp=top[i];
          top[i]=top[j];
          top[j]=temp;
        }
				/* there may be not enuogh literals */
      if( (stt->binarypos[top[i]]==0) && (stt->binaryneg[top[i]]==0 ) ) {
        k=i;
        break;
      }
    }


				/* print the best k variables */

    #ifdef DEBUG
      printf("Best literals: ");
      for(i=0; i<k; i++)
        printf("%d (%d,%d)  ",top[i],stt->binarypos[top[i]],stt->binaryneg[top[i]]);
      printf("\n");
      fflush(NULL);
    #endif
  }
				
				/* now, if k=0 then there are no binary clause in the theory, which
				means we have to call some other procedure to determine the next.
				if k=1 it means that either the number of literals to propagate is
				equal to 1, or that we do not want to propagate literals: in this case,
				we return the maximal literal according to the metrics COMBINE */


  if( k<=0 ) 
    return 0;			/* there are no binary clauses in the theory */
  else if( k==1 )
    bestvar=CHOICESIGN(stt->binarypos[top[0]],stt->binaryneg[top[0]])*top[0];
				/* either k=1, or we just want the maximum */
  else {
				/* evaluate the number of literal deduced by up from each literal */

    #ifdef DEBUG
      printf("UP: ");
    #endif

    bestval=-1;

    for(i=0; i<k; i++) {

				/* do not consider already assigned variables */
      if( stt->val[top[i]]!=0 )
        if( i==0 ) {
          printf("Choice found no unassigned variable\n");
          exit(1);
        }
        else
          break;
  
      #ifdef DEBUG
        printf("%d (",top[i]);
      #endif

				/* positive literal */

      if( stt->binaryneg[top[i]]==0 ) 
        pos=0;
      else {
				/* check number of deduced literals */
        addlevel( stk, top[i], 0, UPOPTIONS );
        rmallqueue(q);
        inqueue(q,top[i]);
        bin=stt->binary;
        u=unitpropagation( q, cl, ix, stt, stk, UPOPTIONS );
        if( maxbin==0 )
          pos=stk->elem[stk->stackindex].numdeduced-1;	/* (of course, top[i] is deduced...) */
        else
          pos=stt->binary-bin;
        rmlevel( cl, ix, stt, stk, UPOPTIONS );
  
        #ifdef DEBUG
          printf("%d",pos);
        #endif
  
  				/* inconsistency/satisfiaction: return literal */
        if( u!=0 ) {
          #ifdef DEBUG
            printf("Choice=%d (top[i]=%d up=%d)\n",u*top[i],top[i],u);
          #endif
          return u*top[i];
        }
      }


      #ifdef DEBUG
        printf(",");
      #endif


				/* negative literal */
      if( stt->binarypos[top[i]]==0 )
        neg=0;
      else {
				/* check number of deduced literals */
        addlevel( stk, -top[i], 0, UPOPTIONS );
        rmallqueue(q);
        inqueue(q,-top[i]);
        bin=stt->binary;
        u=unitpropagation( q, cl, ix, stt, stk, UPOPTIONS );
        if( maxbin==0 )
          neg=stk->elem[stk->stackindex].numdeduced-1;
        else
          neg=stt->binary-bin;
        rmlevel( cl, ix, stt, stk, UPOPTIONS );

        #ifdef DEBUG
          printf("%d",neg);
        #endif

				/* inconsistency/satisfiaction: return literal */
        if( u!=0 ) {
          #ifdef DEBUG
            printf("Choice=%d (-top[i]=%d up=%d)\n",-u*top[i],-top[i],u);
          #endif
          return -u*top[i];
        }
      }

      #ifdef DEBUG
        printf(") ");
        fflush(NULL);
      #endif

				/* if literal is best than the current best, swap */
      if( COMBINE(pos,neg) > bestval ) {
        bestval=COMBINE(pos,neg);
        bestvar=CHOICESIGN(pos,neg)*top[i];
      }
    }


  }

  #ifdef DEBUG
    printf("Choice=%d\n",bestvar); fflush(NULL);
  #endif

  return bestvar;
}

/************************** End Propagation Choice **************************/



/************************** Propagation Choice 2 ****************************/
/*
  Maximize the number of propagated literals. Optimized version.
*/

int top[MAXVAR];
int posded[MAXVAR+1];
int negded[MAXVAR+1];

/* 
  What happens when a literal is propagated?
If a literal x implies n literals, and y is among them, then y alone can
at most produce n literals by unit propagation. This may be useful,
because it gives an upper bound on the measure of y without propagating y.
*/

int resultofpropagating( setofclauses cl, clauseindex ix, state stt, stack stk, int lett, int *posneg ) {
  int i;
  int u;

  addlevel( stk, lett, 0, UPOPTIONS );				/* add level */

  rmallqueue(q);					/* put literal in queue and propagate */
  inqueue(q,lett);

  u=unitpropagation( q, cl, ix, stt, stk, UPOPTIONS );

  *posneg=stk->elem[stk->stackindex].numdeduced-1;	/* this is the number of propagated literals (minus the literal) */

							/* udpate the upper bound for all deduced literals */
  for(i=stk->elem[stk->stackindex].firstdeduced; i<stk->elem[stk->stackindex].numdeduced; i++)
    if( stk->deduced[i]>0 )
      posded[stk->deduced[i]]=min(posded[stk->deduced[i]],*posneg);
    else
      negded[-stk->deduced[i]]=min(negded[-stk->deduced[i]],*posneg);

  rmlevel( cl, ix, stt, stk, UPOPTIONS );				/* remove the level */

  return 1;
}


/*
  This is the function of choice using the number of propagated literals. Note that
the maxbin choice cannot be used here.
*/

int propchoice2( setofclauses cl, clauseindex ix, state stt, stack stk ) {
  int k;
  int i,j;
  int temp;
  int bestval,bestvar;
  int pos,neg;
  int u;

				/* number of literals to propagate */
  k=NUMPROP;
  if( k<1 )
    k=1;

  #ifdef DEBUG
    printf("ChoiceK=%d Binary=%d\n",k,stt->binary); fflush(NULL);
  #endif

				/* determine the top k variables */

  if( k>=1 ) {

    for(i=0; i<cl->numvar; i++)
      top[i]=i+1;

    for(i=0; i<k; i++) {
				/* determine the i-th best */
      for(j=i+1; j<cl->numvar; j++)
        if( (
              ( COMBINE(stt->binarypos[top[i]],stt->binaryneg[top[i]]) <
	        COMBINE(stt->binarypos[top[j]],stt->binaryneg[top[j]]) ) &&
	      ( stt->val[top[j]]==0 )
            ) ||
            ( ( stt->val[top[i]]!=0 ) && ( stt->val[top[j]]==0) )
          ) {
          temp=top[i];
          top[i]=top[j];
          top[j]=temp;
        }
				/* there may be not enuogh literals */
      if( (stt->binarypos[top[i]]==0) && (stt->binaryneg[top[i]]==0 ) ) {
        k=i;
        break;
      }
    }


				/* print the best k variables */

    #ifdef DEBUG
      printf("Best literals: ");
      for(i=0; i<k; i++)
        printf("%d (%d,%d)  ",top[i],stt->binarypos[top[i]],stt->binaryneg[top[i]]);
      printf("\n");
      fflush(NULL);
    #endif
  }
				
				/* now, if k=0 then there are no binary clause in the theory, which
				means we have to call some other procedure to determine the choice.
				if k=1 it means that either the number of literals to propagate is
				equal to 1, or that we do not want to propagate literals: in this case,
				we return the maximal literal according to the COMBINE metrics */


  if( k<=0 ) 
    return 0;			/* there are no binary clauses in the theory */
  else if( k==1 )
    bestvar=CHOICESIGN(stt->binarypos[top[0]],stt->binaryneg[top[0]])*top[0];
				/* either k=1, or we just want the maximum */
  else {
				/* evaluate the number of literal deduced by up from each literal */

    #ifdef DEBUG
      printf("UP: ");
    #endif

				/* set the initial upper bound for each literal */

    for(i=0; i<k; i++) {
      posded[top[i]]=cl->numvar+1;
      negded[top[i]]=cl->numvar+1;
    }

    bestval=-1;

    for(i=0; i<k; i++) {

				/* do not consider already assigned variables */
      if( stt->val[top[i]]!=0 )
        if( i==0 ) {
          printf("Choice found no unassigned variable\n");
          exit(1);
        }
        else
          break;


				/* do not consider variables for which we already know they are not optimal */

      if( COMBINE(posded[top[i]],negded[top[i]]) <= bestval )
        continue;
  

      #ifdef DEBUG
        printf("%d (",top[i]);
      #endif

				/* positive literal */

      if( stt->binaryneg[top[i]]==0 ) 
        pos=0;
      else {
				/* check number of deduced literals */
     
        u=resultofpropagating(cl,ix,stt,stk,top[i],&pos);
  
        #ifdef DEBUG
          printf("%d",pos);
        #endif
  
  				/* inconsistency/satisfiaction: return literal */
        if( u!=0 ) {
          #ifdef DEBUG
            printf("Choice=%d (top[i]=%d up=%d)\n",u*top[i],top[i],u);
          #endif
          return u*top[i];
        }
      }


      #ifdef DEBUG
        printf(",");
      #endif


				/* negative literal */
      if( stt->binarypos[top[i]]==0 )
        neg=0;
      else {
				/* check number of deduced literals */
        u=resultofpropagating(cl,ix,stt,stk,-top[i],&pos);

        #ifdef DEBUG
          printf("%d",neg);
        #endif

				/* inconsistency/satisfiaction: return literal */
        if( u!=0 ) {
          #ifdef DEBUG
            printf("Choice=%d (-top[i]=%d up=%d)\n",-u*top[i],-top[i],u);
          #endif
          return -u*top[i];
        }
      }

      #ifdef DEBUG
        printf(") ");
        fflush(NULL);
      #endif

				/* if literal is better than the current best, swap */
      if( COMBINE(pos,neg) > bestval ) {
        bestval=COMBINE(pos,neg);
        bestvar=CHOICESIGN(pos,neg)*top[i];
      }
    }


  }

  #ifdef DEBUG
    printf("Choice=%d\n",bestvar); fflush(NULL);
  #endif

  return bestvar;
}

/************************** End Propagation Choice **************************/



/*
  Propagation choice test: run propchoice and propchoice2, and if
the result is different, terminate.
*/

propchoicetest( setofclauses cl, clauseindex ix, state stt, stack stk ) {
  int s1,s2;

  s1=propchoice( cl, ix, stt, stk, 0 );
  s2=propchoice2( cl, ix, stt, stk );

  if( s1!=s2 ) {
    printf("Error: propchoice=%d, propchoice2=%d\n",s1,s2);
    exit(1);
  }

  return s1;
}


/***************************** Occurence Choice *****************************/
/*
  Choose the literal appearing most often in the theory.
Do not combine positive and negative count.
*/

int occurrencechoice( setofclauses cl, clauseindex ix, state stt, stack stk ) {
  int bestval, bestvar;
  listofint l;
  int i,pos,neg;

  bestval=-1;

  for(i=1; i<=cl->numvar; i++)
    if( stt->val[i]==0 ) {

      pos=0;                  /* positive occurrences of literal */
      for(l=ix->positive[i]; l!=NULL; l=l->next)
        if( stt->truelits[l->elem]==0 )
          pos++;

      neg=0;                  /* negative occurrences */
      for(l=ix->negative[i]; l!=NULL; l=l->next)
        if( stt->truelits[l->elem]==0 )
          neg++;
                              /* check if we got a new best literal */
      if( pos+neg>bestval ) {         /* COMBINE(pos,neg)>bestval ) { */
        bestval=pos+neg;              /* COMBINE(pos,neg); */
        bestvar=CHOICESIGN(pos,neg)*i;
      }
    }

  return bestvar;
}

/*************************** End Occurence Choice ***************************/


/**************************** Occurence Choice 2 ****************************/
/*
  Choose the literal appearing most often in the theory.
Combine positive and negative count: this way, we try not
to answer with a simplifiable literal.
*/

int occurrencechoice2( setofclauses cl, clauseindex ix, state stt, stack stk ) {
  int bestval, bestvar;
  listofint l;
  int i,pos,neg;

  bestval=-1;

  for(i=1; i<=cl->numvar; i++)
    if( stt->val[i]==0 ) {

      pos=0;                  /* positive occurrences of literal */
      for(l=ix->positive[i]; l!=NULL; l=l->next)
        if( stt->truelits[l->elem]==0 )
          pos++;

      neg=0;                  /* negative occurrences */
      for(l=ix->negative[i]; l!=NULL; l=l->next)
        if( stt->truelits[l->elem]==0 )
          neg++;

                              /* check if we got a new best literal */
      if( COMBINE(pos,neg)>bestval ) { 
        bestval=COMBINE(pos,neg);
        bestvar=CHOICESIGN(pos,neg)*i;
      }
    }

  return bestvar;
}

/*************************** End Occurence Choice ***************************/


/******************************* Choice *************************************/
/*
  This is the function that chooses the next literal. It is based on
the principle of running three functions in sequence: if the first
one returns a value different from 0, this value is returned. If it
returns 0, then the second function is run, and so on. An error is
generated if the last function returns 0.
*/

int choice( setofclauses cl, clauseindex ix, state stt, stack stk, int options ) {
  int s;

  if( options & TRIVIALCHOICE ) {
    s=trivialchoice( cl, ix, stt, stk );
    if( s!=0 )
      return s;
  }
  else if( options & WEIGHTCHOICE ) {
    s=weightchoice( cl, ix, stt, stk );
    if( s!=0 )
      return s;
    s=trivialchoice( cl, ix, stt, stk );
    if( s!=0 )
      return s;
  }
  else if( options & CHOICETEST ) {
    s=propchoicetest( cl, ix, stt, stk );
    if( s!=0 )
      return s;
    s=occurrencechoice( cl, ix, stt, stk );
    if( s!=0 )
      return s;
  }
  else {
    s=propchoice2( cl, ix, stt, stk );
    if( s!=0 )
      return s;
    s=occurrencechoice( cl, ix, stt, stk );
    if( s!=0 )
      return s;
  }

  printf("Error: three wise men return 0.\n");
  exit(1);
}

/***************************** End Choice ***********************************/

