/*
  Functions for building a set of clauses.
*/

#include"list.h"
#include"queue.h"
#include"mindp.h"


				/* new set of clauses */

setofclauses newsetofclauses() {
  setofclauses cl;

  cl=(setofclauses) malloc( sizeof(*cl) );

  if( cl==NULL ) {
    fprintf(stderr,"Malloc failed in newclauses()\n");
    exit(1);
  }

  return cl;
}


				/* init a set of clauses (this is not done by the previous function) */

int initsetofclauses( setofclauses *cl ) {
  (*cl)->numvar=0;
  (*cl)->numclauses=0;
  (*cl)->cl[0].numlett=0;
  (*cl)->cl[0].lett=NULL;

  return 0;
}


				/* end a clause */

int endclause( setofclauses *cl ) {
  if( (*cl)->numclauses >= MAXCLAUSES ) {
    fprintf(stderr,"Too many clauses.\n");
    exit(1);
  }

  if( (*cl)->cl[(*cl)->numclauses].numlett == 0 )
    return -1;
  else {
    (*cl)->numclauses++;
    (*cl)->cl[(*cl)->numclauses].numlett=0;
    (*cl)->cl[(*cl)->numclauses].lett=NULL;

    return 0;
  }
}


				/* add a literal to the current clause */

int addliteral( int lit, setofclauses *cl ) {
  if( abs(lit)>(*cl)->numvar )
    if( abs(lit)<=MAXVAR )
      (*cl)->numvar=abs(lit);
    else {
      fprintf(stderr,"Too many variabiles.\n");
      exit(1);
    }

  (*cl)->cl[(*cl)->numclauses].numlett++;
  (*cl)->cl[(*cl)->numclauses].lett=conslistofint( (*cl)->cl[(*cl)->numclauses].lett, lit );

  return 0;
}


				/* free a set of clauses */

int freeclauses( setofclauses cl ) {
  int c;

  for(c=0; c<cl->numclauses; c++)
    freelistofint( cl->cl[c].lett );

  free(cl);

  return 0;
}


				/* set weight of variable x */

int setweight( int x, int w, setofclauses *cl ) {
  (*cl)->weight[x]=w;
 
  return 0;
}


				/* value of a variable in the model */

int valueofvar( int x, state stt ) {
  return stt->val[x];
}


				/* free a state */

void freestate( state stt ) {
  free(stt);
}


				/* free a stack */

void freestack( stack stk ) {
  free(stk);
}


				/* free an index */

void freeindex( clauseindex ix ) {
  int i;
  
  for(i=1; i<=ix->numvar; i++) {
    freelistofint( ix->positive[i] );
    freelistofint( ix->negative[i] );
  }

  free(ix);
}


