/*
  Functions for copying states, stacks, clauses.
*/

#include"list.h"
#include"queue.h"
#include"mindp.h"



				/* copy a state into another */

int copystate( state st1, state st2 ) {
  int i;
  
  st1->numvar=st2->numvar;
  st1->numclauses=st2->numclauses;

  memcpy( st1->val, st2->val, (st2->numvar+1)*sizeof(char) );

  memcpy( st1->truelits, st2->truelits, st2->numclauses*sizeof(int) );
  memcpy( st1->falselits, st2->falselits, st2->numclauses*sizeof(int) );

  memcpy( st1->poslits, st2->poslits, st2->numclauses*sizeof(int) );

  memcpy( st1->pos, st2->pos, (st2->numvar+1)*sizeof(int) );
  memcpy( st1->neg, st2->neg, (st2->numvar+1)*sizeof(int) );

  st1->posclause=newpartition();
  copypartition( st1->posclause, st2->posclause );

  st1->binary=st2->binary;

  memcpy( st1->binarypos, st2->binarypos, (st2->numvar+1)*sizeof(int) );
  memcpy( st1->binaryneg, st2->binaryneg, (st2->numvar+1)*sizeof(int) );

  memcpy( st1->posocc, st2->posocc, (st2->numvar+1)*sizeof(int) );

  st1->weighted=st2->weighted;

  st1->found=st2->found;
  st1->kmax=st2->kmax;
  st1->kmin=st2->kmin;

  st1->branch=st2->branch;
  st1->intertime=st2->intertime;
  st1->intermod=st2->intermod;

  return 0;
}


				/* copy a stack into another */

int copystack( stack stk1, stack stk2 ) {
  int l,x,i;

  l=stk2->stackindex;

  stk1->stackindex=l;
  
  for(x=0; x<=l; x++) {
    stk1->elem[x].literal=stk2->elem[x].literal;
    stk1->elem[x].triedneg=stk2->elem[x].triedneg;

    stk1->elem[x].firstdeduced=stk2->elem[x].firstdeduced;
    stk1->elem[x].numdeduced=stk2->elem[x].numdeduced;

    stk1->elem[x].weight=stk2->elem[x].weight;

    stk1->elem[x].numtrueclauses=stk2->elem[x].numtrueclauses;

    for(i=stk2->elem[x].firstdeduced; i<stk2->elem[x].firstdeduced+stk2->elem[x].numdeduced; i++)
      stk1->deduced[i]=stk2->deduced[i];
  }

  return 0;
}

				/* copy a set of clauses to a new one: cla=clb */

int copyclauses( setofclauses cla, setofclauses clb ) {
  listofint l;
  int c;

			/* copy the clauses */
  for(c=0; c<clb->numclauses; c++) {
    l=cla->cl[c].lett;
    cla->cl[c].lett=clb->cl[c].lett;
    freelistofint(l);
    cla->cl[c].numlett=clb->cl[c].numlett;
  }

			/* free the other elements (if any) */
  for(c=clb->numclauses; c<cla->numclauses; c++)
    freelistofint(cla->cl[c].lett);

			/* new number of clauses */
  cla->numclauses=clb->numclauses;

  return 0;
}


