/*
  All the I/O functions are here
*/

#include"list.h"
#include"queue.h"
#include"mindp.h"

#define noSEEINPUT

				/* input a set of clauses */

int inputclausesonly( FILE *stream, setofclauses *cl ) {
  int c,v,n;
  char line[10000];
  char *current;

  fscanf(stream,"%[ \n]s",line);

  initsetofclauses( cl );

  while( fscanf(stream,"%[^\n]s",line)!=EOF ) {
    if( *line!='\0' ) {

      #ifdef SEEINPUT
        printf("%s\n",line);
      #endif

      if( *line=='%' )
        return -1;
      
      if( *line==':' )
        return 0;

      if( (*line!='p') && (*line!='c') ) {

        current=line;
      
        while( (sscanf(current,"%d",&v) ) && (*current!='\0') ) {
          if( v==0 )
            break;

          addliteral(v, cl);

          while( (*current=='-') || isdigit(*current) ) current++;
          while( ( (*current==' ') || (*current=='\t') ) && (*current!='\0') )
            current++;
        }

        endclause(cl);
      }
    }
    
    if( fscanf(stream,"%[\n]s",line)==EOF )
      break;

  }

  return 0;
}


				/* input a set of weight */

int inputweightonly( FILE *stream, setofclauses *cl ) {
  int x;
  char word[200];

  for(x=1; x<=(*cl)->numvar; x++)
    if( fscanf(stream,"%s",word)==EOF )
      break;
    else if( *word=='%' )
      break;
    else
      setweight(x, atoi(word), cl);

  return 0;
}


				/* input clauses & weight */

int readclauses( FILE *stream, setofclauses *cl ) {
  int x;

  initsetofclauses(cl);

  if( inputclausesonly( stream, cl )>=0 ) {
    for(x=1; x<(*cl)->numvar; x++)
      (*cl)->weight[x]=0;

    inputweightonly( stream, cl );
  }

  return 0;
}

				/* print the set of clauses */

int printclauses( setofclauses cl ) {
  int c;
  listofint l,r;

  printf("Number of variables=%d\n",cl->numvar);
  printf("Number of clauses=%d\n",cl->numclauses);
  for(c=0;c<cl->numclauses;c++) {
    // printf("%d (%d): ",c,cl->cl[c].numlett);
    r=reverselistofint(cl->cl[c].lett);
    printlistofint(stdout,r);
    freelistofint(r);
    printf("0\n");
  }

  return 0;
}

int printclausesweight( setofclauses cl ) {
  int x;

  printclauses(cl);

  printf("Weights:\n");
  for(x=1; x<=cl->numvar; x++)
    printf("%d: %d\n",x,cl->weight[x]);
  printf("\n");

  return 0;
}


				/* print a clause only */

int printclause( setofclauses cl, int c ) {
  listofint l;

  for( l=cl->cl[c].lett; l!=NULL; l=l->next )
    printf("%d ",l->elem);
  printf("\n");

  return 0;
}


				/* print the index of the clauses */

int printindex( clauseindex ix ) {
  int v;
  listofint l;

  for(v=1;v<=ix->numvar;v++) {
    printf("%5d:",v);
    for(l=ix->positive[v]; l!=NULL; l=l->next)
      printf("%5d ",l->elem);
    printf("\n");

    printf("%5d:",-v);
    for(l=ix->negative[v]; l!=NULL; l=l->next)
      printf("%5d ",l->elem);
    printf("\n");
  }
    
  return 0;
} 


				/* print the partial model */

int printmodel( state s ) {
  int i;

  for(i=1; i<=s->numvar; i++)
    if( s->val[i] )
      printf("%d ",s->val[i]*i );
    else
      printf("%d ",-i);
      /* printf("+/-%d ",i); */

  printf("\n");

  return 0;
}


				/* print the current evaluation of the variables
				   whose weight is not zero */

int printweighted( setofclauses cl, state stt ) {
  int i;

  for(i=1; i<=cl->numvar; i++)
    if( cl->weight[i] ) {
      printf("%d=",i);
      if( stt->val[i]==0 )
        printf("*");
      else
        printf("%d",stt->val[i]);
      printf(" ");
    }
  printf("\n");

  return 0;
}


				/* print the positive literals of a model */

int printpositive( state stt ) {
  int i;

  for(i=1; i<=stt->numvar; i++)
    if( stt->val[i]==1 )
      printf("%d ",i);

  printf("\n");

  return 0;
}


				/* print the state */

int printstate( state s ) {
  int i;

  for(i=1; i<=s->numvar; i++)
  if( s->val[i] )
    printf("%3d ",s->val[i]*i );
  printf("\n");

/*
  printf("Clause count: ");
  for(i=0; i<s->numclauses; i++)
    printf("%d=(%d:%d,%d) ",i,cl->cl[i].numlett,s->truelits[i],s->falselits[i]);
  printf("\n");
*/

  return 0;
}


				/* print a stack */

int printstack( stack stk ) {
  int i,x;

  printf("Literals: [ ");
  for(i=1; i<=stk->stackindex; i++)
    printf("%d ",stk->elem[i].literal);
  printf("]\n");

  printf("Deduced literals: [ ");
  for(i=1; i<=stk->stackindex; i++) {
    printf("{ ");
    for(x=stk->elem[i].firstdeduced; x<stk->elem[i].firstdeduced+stk->elem[i].numdeduced; x++)
      printf("%d ",stk->deduced[x]);
    printf("} ");
  }
  printf("]\n");

  printf("Numtrueclauses: [ ");
  for(i=1; i<=stk->stackindex; i++)
    printf("%d ",stk->elem[i].numtrueclauses);
  printf("]\n");

  printf("\n");

  return 0;
}
