/*
  Header file for the DP functions.
list.h should be included before this
file.
*/

#include"limits.h"
#include"partition.h"

#define NONPOSCLAUSE 0
#define POSCLAUSE 1

                /* sign of an integer */

#define sgn(x)  (((x)>0)?1:-1)
#define pos(x)  (((x)>0)?1:0)

#define max(x,y) (((x)>(y))?(x):(y))


		/* one clauses */

typedef struct {
  int numlett;				/* number of literals */
  listofint lett;			/* list of the literals */
} clause;


		/* a set of clauses */

typedef struct {
  int numvar;
  int weight[MAXVAR+1];

  int numclauses;
  clause cl[MAXCLAUSES];

  void *userfield;			/* user defined functions...? */
} *setofclauses;


		/* index of the clauses: everything static used for an easy access to the clauses */

typedef struct {
  int numvar;
  int numpositive[MAXVAR+1];
  listofint positive[MAXVAR+1];		/* clauses in which each variable appears positive */
  int numnegative[MAXVAR+1];
  listofint negative[MAXVAR+1];		/* clauses in which each variable appears negated */

  int numcand;
  int cand[MAXCLAUSES];			/* number and set of candidate clauses (see README) */

  int weightorder[MAXVAR+1];		/* variables in weight decreasing order */
} *clauseindex;


		/* stato corrente */

typedef struct {
  int numvar;
  int numclauses;

  char val[MAXVAR+1];			/* -1/0/1 depending on the value, 0 is undetermined */

  int truelits[MAXCLAUSES];		/* number of true and false literals in each clause */
  int falselits[MAXCLAUSES];

  int pos[MAXVAR+1];                    /* number of clauses in which a literal appears */
  int neg[MAXVAR+1];

  int poslits[MAXCLAUSES];		/* number of positive weighted unassigned literals in clause */

  partition *posclause;			/* partition of clauses: positive and non-positive
					  (positive=all unassigned literals have weight>0, no true lit) */

  int binary;
  int binarypos[MAXVAR+1];		/* number of binary clauses in which a literal appears */
  int binaryneg[MAXVAR+1];

  int posocc[MAXVAR+1];			/* number of positive clauses in which a literal occour */

  int weighted;				/* is the set weighted or not? */

  int found;				/* has the first model been found? */
  int kmax;				/* upper bound of the weight */
  int kmin;				/* lower bound of the weight */

  listofint intermod;
  listofint intertime;
  int branch;				/* branching nodes seen so far */
} *state;


		/* one element of the stack */

typedef struct {
  int literal;				/* literal tried at this level */
  int triedneg;				/* has the negated been already tried or not? */

  int firstdeduced;			/* index of the first deduced literal at this level of the stack */
  int numdeduced;			/* number of deduced literals */

  int weight;				/* weight of the current assignment */

  int firstunassweighted;		/* first (possible) unassigned literal in order of weight */

  int numtrueclauses;			/* number of clauses currently satisfied */
} stackelem;


		/* stack */

typedef struct {
  stackelem elem[MAXVAR+1];
  int deduced[MAXVAR+1];		/* this is the stack of literals deduced so far */
  int stackindex;			/* index of the stack */
} *stack;


		/* functions for buiding a set of clauses */

setofclauses newsetofclauses();				/* allocate a set of clauses */
int initsetofclauses( setofclauses *cl );			/* init a set of clauses */
int endclause( setofclauses *cl );			/* end the current clause */
int addliteral( int lit, setofclauses *cl );		/* add literal to the current clause */

		/* free structures */

int freesetofclauses( setofclauses cl );
void freestate( state stt );
void freestack( stack stk );
void freeindex( clauseindex ix );

		/* value of a variable in a model */

int valueofvar( int x, state stt );


		/* I/O functions */

int readclauses( FILE *stream, setofclauses *cl );	/* read clauses & weights from stdin */
int printclauses( setofclauses cl );			/* print clauses & weight */
int printclause( setofclauses cl, int c );		/* print a clause */
int printindex( clauseindex ix );
int printmodel( state s );
int printweighted( setofclauses cl, state stt );
int printpositive( state stt );
int printstate( state s );
int printstack( stack stk );
void printstatus();
int usertime();


		/* options (bit 0 is not used) */

#define SIMPLIFY	(1<<1)
#define PROPAGATE 	(1<<2)
#define TRIVIALCHOICE 	(1<<3)
#define WEIGHTCHOICE 	(1<<4)
#define PROPCHOICE 	(1<<5)

#define SELFTEST 	(1<<8)
#define CHOICETEST 	(1<<9)

#define POSITIVE 	(1<<10)
#define OVER		(1<<11)

#define trattoreDEFAULTDP	( SIMPLIFY | PROPAGATE | TRIVIALCHOICE | SELFTEST | CHOICETEST | POSITIVE | OVER )
#define DEFAULTDP	( SIMPLIFY | PROPAGATE | POSITIVE | OVER )


		/* check satisfiability */

int sat( setofclauses cl, int options );

		/* check sat, and return one model */

int onemodel( setofclauses cl, state *stt, int options );

		/* first model */

int firstmodel( setofclauses cl, clauseindex *ix, state *stt, stack *stk, int options );

		/* next model */

int nextmodel( setofclauses cl, clauseindex ix, state stt, stack stk, int options );


		/* first minimal model only */

int minweight( setofclauses cl, state *sttlast, int options );

		/* first minimal model */

int firstminweight( setofclauses cl, clauseindex *ix, state *sttlast, stack *stklast, int options );

		/* next minimal model */

int nextminweight( int kmax, setofclauses cl, clauseindex ix, state stt, stack stk, int options );


		/* initial */

int initial( setofclauses cl, clauseindex *ix, state *stt, stack *stk, int options );

		/* next literal to expand */

int choice( setofclauses cl, clauseindex ix, state stt, stack stk, int options );

		/* unit propagation */

int unitpropagation( queue *q, setofclauses cl, clauseindex ix, state stt, stack stk, int options );

		/* add a level to the stack */

int addlevel( stack stk, int lit, int tried, int options );

		/* remove a level from the stack */

int rmlevel( setofclauses cl, clauseindex ix, state stt, stack stk, int options );

		/* check integrity of dynamically built strucutres */

int selftest( setofclauses cl, clauseindex ix, state stt, stack stk, int options );

int consselftest( setofclauses cl, clauseindex ix, state stt, stack stk, int options );


		/* simplify */

int simplify( setofclauses cl, clauseindex ix, state stt, stack stk, int options );


		/* lower bound */

int allclauseslowerbound( setofclauses cl, clauseindex ix, state stt, int sign );
int candidatelowerbound( setofclauses cl, clauseindex ix, state stt, int sign );
int poscountlowerbound( setofclauses cl, clauseindex ix, state stt, int sign );
int weightlowerbound( setofclauses cl, clauseindex ix, state stt, int sign, int options );

