#include<stdlib.h>
#include<stdio.h>
#include<time.h>

#define MAXNUMCLAUSOLE 1000
#define MAXNUMVAR 100
#define MAXNUMLETT 1000
#define MAXMODELLI 30000

/* #define DEBUG */

int numvar;
int numclausole;
int clausole[MAXNUMCLAUSOLE][MAXNUMLETT];
int lungcl[MAXNUMCLAUSOLE];


long starttime, endtime;

int pesi[MAXNUMVAR+1];


	/* valore della variabile di un modello */

int varofmodel( int var, long modello ) {
  return (modello>>(var-1))&1;
}



	/* stampa un modello */

void printmodel( long modello ) {
  int j;

  printf("[");
  for(j=1; j<=numvar; j++)
    printf("%d",varofmodel(j,modello));
  printf("]\n");
}



	/* verifica se una interpretazione e' un modello di un insieme di clausole */

int ismodel( long modello ) {
  int c,v;
  int satset,satcla;

  satset=1;

  for(c=0; c<numclausole; c++) {
    satcla=0;
    for(v=0; v<lungcl[c]; v++) {
      if( clausole[c][v] > 0 ) {
        if( varofmodel(clausole[c][v],modello) )
          satcla=1;
      }
      else
        if( !varofmodel(-clausole[c][v],modello) )
          satcla=1;
    }
    if( satcla==0 )
      satset=0;
  }

  return satset;
}



		/* cardinalita' di un modello */

int cardinality( long modello )
{
  int c=0;
  int i;

  for(i=1; i<=numvar; i++)
    if( varofmodel(i,modello) )
      c+=pesi[i];

  return c;
}


		/* contenimento fra modelli */

int containment( long modello1, long modello2 )
{
  if( modello1 & ~modello2 )
    return 0;
  else
    return 1;
}


		/* modelli minimi */

int minimi( int nummodelli, long modelli[1000], long modelliminimi[1000] ) {
  int numminimi=0;
  int i,j;
  int c;

  for( i=0; i<nummodelli; i++ ) {
    c=1;
    for( j=0; j<nummodelli; j++ ) {
      if( ( containment( modelli[j], modelli[i] ) ) && (i!=j) ) {
        c=0;
        j=nummodelli;
      }
    }
    if( c ) {
      modelliminimi[numminimi]=modelli[i];
      numminimi++;
    }
  }

  return numminimi;
}
      


main( int argc, char* argv[] ) {
  int c,v,x;
  int modello;
  int i;
  int nummodelli=0;
  long modelli[MAXMODELLI];
  int mincard;
  int app;
 


			/* lettura clausole */

  scanf("%d %d",&numvar,&numclausole);
  for(c=0; c<numclausole; c++) {
    v=0;
    do {
      scanf("%d",&clausole[c][v]);
      v++;
    } while( clausole[c][v-1]!=0 );
    lungcl[c]=v-1;
  }


			/* toglie le variabili che non appaiono */

  for(i=numvar; i>=1; i--) {
    app=0;
    for(c=0; c<numclausole; c++)
      for(v=0; v<lungcl[c]; v++)
        if( abs(clausole[c][v])==i )
          app=1;

    if( app==0 )
      numvar--;
    else
      break;
  }

			/* lettura pesi */

  for(v=1;v<=numvar;v++)
    if( scanf("%d",&i)!=EOF )
      pesi[v]=i;
    else
      pesi[v]=0;


			/* stampa clausole */

#ifdef DEBUG
  printf("\nnumvar=%d\nnumclausole=%d\n",numvar,numclausole);
  printf("Clausole:\n");
  for(c=0; c<numclausole; c++) {
    for(v=0; v<lungcl[c]; v++)
      printf("%2d ",clausole[c][v]);
    printf("\n");
  }

  printf("Weight:\n");
  for(v=1;v<=numvar;v++)
    printf("%d: %d\n",v,pesi[v]);
#endif


			/* calcolo esaustivo modelli */
  starttime=clock();

  mincard=0;
  for(i=1;i<=numvar;i++)
    mincard+=pesi[i];

  for(modello=0; modello<(1<<numvar); modello++) {
    if( ismodel(modello) ) {
      if( cardinality(modello)<mincard ) 
        mincard=cardinality(modello);
      modelli[nummodelli]=modello;
      nummodelli++;
      if( nummodelli>MAXMODELLI ) {
        printf("\nTroppi modelli.");
        exit(1);
      }
    }
  }

  if( nummodelli>0 )
    printf("Consistente: peso=%d\n",mincard);
  else
    printf("Inconsistente\n");

  for(i=0; i<nummodelli; i++) {
    if( cardinality(modelli[i])==mincard )
      printmodel(modelli[i]);
  }

  endtime=clock();
  
  return 0;
}


