/*
  Stampa anche il primo modello minimo

*/


/*
  Minimal models using Davis-Putnam
Heuristics:
1. fail whenever the weight of a model is greater or equal than the last best found
2. use positive clauses to infer a lower bound of the weight
2. all the variables whose weight is more than the current difference are set to false

Note: it must hold weight*1000 < overflow.

Define:

USESIGNALS	compile with the functions for handling signals

*/

#include"dp.h"

#include<signal.h>
#include<sys/times.h>

#define PIDFILENAME "pid.mindp"

int usertime() {
  struct tms temp;

  times(&temp);
  return temp.tms_utime;
}

                                /* signal handlers */

void onexit() {
  unlink(PIDFILENAME);
}

#ifdef USESIGNALS

  void usersignal( int s ) {
    printstatus();

    signal( SIGUSR1, usersignal);
  }

  void exitsignal( int s ) {
    printf("\nSignal %d caught. Exiting.\n",s);
    printstatus(s);
    onexit();

    exit(1);
  }
#endif


main( int argc, char *argv[] ) {
  int c;
  setofclauses cl;
  clauseindex ix;
  state stt;
  stack stk;
  int k;
  int i;

  FILE *pidfile;
  int starttime, leaptime, endtime;


  cl=newsetofclauses();
  readclauses( stdin, &cl );

  /* printclauses( cl ); */

  #ifdef DEBUG
    printclauses( cl );
  #endif

  #ifdef USESIGNALS
    pidfile=fopen(PIDFILENAME,"w");
    fprintf(pidfile,"%d",getpid());
    fclose(pidfile);
  
    signal( SIGTERM, exitsignal );
    signal( SIGINT, exitsignal );
    signal( SIGUSR1, usersignal );
  #endif

  if( argc>1 ) {
    if( onemodel(cl,&stt,DEFAULTDP)==1 )
      printf("SAT\n");
    else
      printf("UNSAT\n");

    printf("%d\n",stt->branch);

    return 0;
  }

  starttime=usertime();
  k=minweight( cl, &stt, DEFAULTDP );
  endtime=usertime();

  if( k<0 )
    printf("Inconsistente\n");
  else {
    printf("Consistente: peso=%d\n",k);
    for(i=1; i<=cl->numvar; i++)
      printf("%d=%d ",i,valueofvar(i,stt));
    printf("\n");
  }


//  printf("%d\n",stt->branch);

//  fprintf(stderr,"nodes: %d\n",stt->branch);
  fprintf(stderr,"%d\n",endtime-starttime);

  onexit();

  return 0;
}

