#include"limits.h"
#include"partition.h"

				/* allocate space for a multiclass */

partition *newpartition() {
  partition *prt;

  prt=(partition *) malloc( sizeof(partition) );

  return prt;
}


				/* all the elements are initially in class 0 */

int initpartition( int numelem, int numclasses, partition *prt ) {
  int i;

  if( prt==NULL ) {
    printf("Initialization of a NULL partition.\n");
    exit(1);
  }

  prt->maxclass=numclasses-1;
  prt->maxelem=numelem-1;

  prt->initclass[0]=0;
  for(i=1; i<numclasses; i++)
    prt->initclass[i]=ENDOFCLASS;

  for(i=0; i<numelem; i++)
    prt->elem[i].class=0;
    
  prt->elem[0].prev=ENDOFCLASS;
  for(i=1; i<numelem; i++) 
    prt->elem[i].prev=i-1;

  for(i=0; i<numelem-1; i++)
    prt->elem[i].next=i+1;
  prt->elem[numelem-1].next=ENDOFCLASS;

  return 0;
}
    

				/* remove element from its class (not to be used outside the module) */

int rmelem( int elem, partition *prt ) {
  if( prt==NULL ) {
    printf("Remove element in a NULL partition.\n");
    exit(1);
  }

  if( elem > prt->maxelem ) {
    printf("Element %d exeeding maximum %d\n",elem,prt->maxelem);
    exit(1);
  }

  if( prt->elem[elem].prev==ENDOFCLASS )
    prt->initclass[ prt->elem[elem].class ]=prt->elem[elem].next;
  else
    prt->elem[prt->elem[elem].prev].next=prt->elem[elem].next;

  prt->elem[prt->elem[elem].next].prev=prt->elem[elem].prev;

  return 0;
}


				/* add element to a class (not to be used directly) */

int addelem( int elem, int class, partition *prt ) {
  if( prt==NULL ) {
    printf("Add element in a NULL partition.\n");
    exit(1);
  }

  if( class > prt->maxclass ) {
    printf("Class %d exeeding maximum %d\n",class,prt->maxclass);
    exit(1);
  }
  if( elem > prt->maxelem ) {
    printf("Element %d exeeding maximum %d\n",elem,prt->maxelem);
    exit(1);
  }

  prt->elem[elem].class=class;

  if( prt->initclass[class]!=ENDOFCLASS )
    prt->elem[ prt->initclass[class] ].prev=elem;

  prt->elem[elem].next=prt->initclass[class];

  prt->initclass[class]=elem;
  prt->elem[elem].prev=ENDOFCLASS;

  return 0;
}


				/* move element to class c */

int moveelem( int elem, int class, partition *prt ) {
  if( prt==NULL ) {
    printf("Move element in a NULL partition.\n");
    exit(1);
  }

  rmelem( elem, prt );
  addelem( elem, class, prt );

  return 0;
}


				/* gives the class of an element */

int whichclass( int elem, partition *prt ) {
  if( prt==NULL ) {
    printf("Class test in a NULL partition.\n");
    exit(1);
  }

  return prt->elem[elem].class;
}


				/* is class empty? */

int isclassempty( int class, partition *prt ) {
  if( prt==NULL ) {
    printf("Check emptyness of a NULL partition.\n");
    exit(1);
  }

  return ( prt->initclass[class] == ENDOFCLASS );
}


				/* first element of a class */

int firstofclass( int class, partition *prt ) {
  if( prt==NULL )
    return -1;

  return prt->initclass[class];
}


				/* next element of a class (given the previous one) */

int nextofclass( int elem, partition *prt ) {
  if( prt==NULL )
    return -1;

  return prt->elem[elem].next;
}


				/* copy a partition */

int copypartition( partition *prt1, partition *prt2 ) {
  int c,e;

  if( (prt1==NULL) || (prt2==NULL) )
    return -1;

  prt1->maxclass=prt2->maxclass;

  for(c=0; c<=prt2->maxclass; c++)
    prt1->initclass[c]=prt2->initclass[c];

  prt1->maxelem=prt2->maxelem;

  for(e=0; e<=prt2->maxelem; e++) {
    prt1->elem[e].class=prt2->elem[e].class;
    prt1->elem[e].prev=prt2->elem[e].prev;
    prt1->elem[e].next=prt2->elem[e].next;
  }
 
  return 0;
}


				/* print all the elements of a class */

int printclass( int class, partition *prt ) {
  int i;

  if( prt==NULL )
    return -1;

				/* example of how loops are done */
  for(i=firstofclass(class,prt); i!=ENDOFCLASS; i=nextofclass(i,prt) )
    printf("%d ",i);

  printf("\n");

  return 0;
}

