/*
  Check the data structures build incrementally
*/

#include"list.h"
#include"queue.h"
#include"mindp.h"


/*
  What to do if a mistake is found.
*/

int mistake() {
  printf("Wrong\n");
//  exit(1); 
  return 0;
}


/*
  Check the positive count of variables and clauses.
*/

int checkpositive( setofclauses cl, clauseindex ix, state stt, stack stk, int sign ) {
  int c,d,pos;
  listofint l;
  int quanti[MAXVAR+1];
  int i;

  #ifdef SDEBUG
    printf("Checking positive count.\n");
  #endif


  memset( quanti, 0, (cl->numvar+1) * sizeof(int) );

                                /* determine the positive clauses and the array
                                   of occurencies of variables */

  for(c=0; c<cl->numclauses; c++) {
    d=c;

    if( stt->truelits[d]==0 ) {

      pos=1;
                                                /* check for negative literals */
      for(l=cl->cl[d].lett; l!=NULL; l=l->next)
        if( ( (sgn(l->elem)==-sign) || (cl->weight[abs(l->elem)]==0) ) &&
            (stt->val[abs(l->elem)]==0) ) {
          pos=0;
          break;
        }

                                                /* add to the count of clauses and literals */
      if( pos ) 
        for(l=cl->cl[d].lett; l!=NULL; l=l->next)
          if( (sgn(l->elem)==sign) && (stt->val[abs(l->elem)]==0) )
            quanti[abs(l->elem)]++;


                                                /* check the positive count of clauses */

      if( (pos==1) && ( whichclass( d, stt->posclause )!=POSCLAUSE ) ) {
        printf("Clause %d is positive but not in posclause\n",d);
        mistake();
      }

      if( (pos==0) && ( whichclass( d, stt->posclause )!=NONPOSCLAUSE ) ) {
        printf("Clause %d is not positive but is in posclause\n",d);
        mistake();
      }

    } /* end if truelits[d]==0 */

  } /* end forall clauses */

                                                /* check the positive occurrencies of variables */

  for(i=1; i<=cl->numvar; i++)
    if( stt->val[i]==0 )
      if( quanti[i]!=stt->posocc[i] ) {
        printf("Positive occurrence of variable %d count wrong: occ=%d, posocc=%d\n",
                i,quanti[i],stt->posocc[i]);
        mistake();
      }

  return 0;
}


/*
  Check the binary count.
*/

int checkbinarycount( setofclauses cl, clauseindex ix, state stt, stack stk ) {
  int positive[MAXVAR+1];
  int negative[MAXVAR+1];
  int i,c;
  listofint l;

  memset( positive, 0, (cl->numvar+1)*sizeof(int) );
  memset( negative, 0, (cl->numvar+1)*sizeof(int) );

  for(c=0; c<cl->numclauses; c++)
    if( (stt->truelits[c]==0) && (cl->cl[c].numlett == stt->falselits[c]+2) )
      for( l=cl->cl[c].lett; l!=NULL; l=l->next )
        if( l->elem > 0 )
          positive[l->elem]++;
        else
          negative[-l->elem]++;

  for(i=1; i<=cl->numvar; i++)
    if( stt->val[i]==0 ) {
      if( positive[i]!=stt->binarypos[i] ) {
        printf("Binary count wrong for variable %d: positive=%d binarypos=%d\n",
                i,positive[i],stt->binarypos[i]);
        mistake();
      }
      if( negative[i]!=stt->binaryneg[i] ) {
        printf("Binary count wrong for variable %d: negative=%d binaryneg=%d\n",
                i,negative[i],stt->binaryneg[i]);
        mistake();
      }
    }

  return 0;
}


/*
  Check occurrence count 
*/

int checkocccount( setofclauses cl, clauseindex ix, state stt, stack stk, int options ) {
  int i;
  int o;
  listofint l;

  if( (options&SIMPLIFY)==0 )
    return 0;

  for(i=1; i<=cl->numvar; i++)
    if( stt->val[i]==0 ) {

      o=0;
      for(l=ix->positive[i]; l!=NULL; l=l->next)
        if( stt->truelits[l->elem]==0 )
          o++;

      if( o!=stt->pos[i] ) {
        printf("Positive count wrong for variable %d: %d instead of %d\n",i,stt->pos[i],o);
        exit(1);
      }

      o=0;
      for(l=ix->negative[i]; l!=NULL; l=l->next)
        if( stt->truelits[l->elem]==0 )
          o++;

      if( o!=stt->neg[i] ) {
        printf("Negative count wrong for variable %d: %d instead of %d\n",i,stt->neg[i],o);
        exit(1);
      }
    }

  return 0;
}


/*
  Check the true/false count
*/

int truefalsecheck( setofclauses cl, clauseindex ix, state stt, stack stk ) {
  int c,i;
  listofint l;
  int t,f;

  for(c=0; c<cl->numclauses; c++) {
    t=0;
    f=0;

    for(l=cl->cl[c].lett; l!=NULL; l=l->next)
      if( l->elem > 0 ) {
        if( stt->val[l->elem]==1 )
          t++;
        else if( stt->val[l->elem]==-1 )
          f++;
      }
      else
        if( stt->val[-l->elem]==-1 )
          t++;
        else if( stt->val[-l->elem]==1 )
          f++;

    if( t!=stt->truelits[c] ) {
      printf("True count wrong for clause %d: %d instead of %d\n",c,stt->truelits[c],t);
      mistake();
    }

    if( f!=stt->falselits[c] ) {
      printf("False count wrong for clause %d: %d instead of %d\n",c,stt->falselits[c],f);
      mistake();
    }

  }

  return 0;
}


/*
  Check the consitency between the model and the stack
*/

int modelcheck( setofclauses cl, clauseindex ix, state stt, stack stk ) {
  char model[MAXVAR+1];
  int i;

  memset( model, 0, (cl->numvar+1)*sizeof(char) );

  for(i=0; i<stk->elem[stk->stackindex].firstdeduced+stk->elem[stk->stackindex].numdeduced; i++) {
    if( model[abs(stk->deduced[i])] != 0 ) {
      printf("Literal implied two times\n");
      mistake();
    }
    if( stk->deduced[i] > 0 )
      model[stk->deduced[i]]=1;
    else
      model[-stk->deduced[i]]=-1;
  }

  for(i=1; i<=cl->numvar; i++)
    if( model[i]!=stt->val[i] ) {
      printf("Model is inconsistent with stack: variable %d is %d (stack) or %d (model)?\n",
             i,model[i],stt->val[i]);
      mistake();
    }

  return 0;
}


/*
  Self test: run all above procedures
*/

int selftest( setofclauses cl, clauseindex ix, state stt, stack stk, int options ) {
  if( (options&POSITIVE) && (stt->weighted) )
    checkpositive( cl, ix, stt, stk, 1 );
  truefalsecheck( cl, ix, stt, stk );
  modelcheck( cl, ix, stt, stk );

  return 0;
}


/*
  This is for the binary/occurrence count, which cannot
be verified after up, as the state is inconsistent after
a failure
*/

int consselftest( setofclauses cl, clauseindex ix, state stt, stack stk, int options ) {
  checkocccount( cl, ix, stt, stk, options );
  checkbinarycount( cl, ix, stt, stk );

  return 0;
}

